import fetch from 'node-fetch';
import SemanticReleaseError from "@semantic-release/error";

export default async (pluginConfig, context) => {
  const { logger, nextRelease, env } = context;
  const { version } = nextRelease;

  const projectPath = env.CI_PROJECT_PATH;
  if (!projectPath) {
    throw new SemanticReleaseError('Could not determine project path. Make sure CI_PROJECT_PATH is set.');
  }

  const apiUrl = env.CI_API_V4_URL || 'https://gitlab.com/api/v4';

  logger.log(`Creating truncated GitLab tag for version ${version}`);

  // Check if we have enough version parts to actually perform truncation
  const versionParts = version.split('.');
  if (versionParts.length < 3) {
    logger.log(`Version ${version} doesn't need truncation (less than 3 parts)`);
    return { truncatedTagCreated: false };
  }

  // Create truncated version tag (e.g., v1.2)
  const truncatedVersion = versionParts.slice(0, 2).join('.');
  const truncatedTagName = `v${truncatedVersion}`;

  await createOrUpdateTag(
    apiUrl,
    projectPath,
    truncatedTagName,
    `Latest ${truncatedTagName}.x release`,
    nextRelease.gitHead,
    env.GITLAB_TOKEN,
    logger
  );

  return { truncatedTagCreated: truncatedTagName };
};

/**
 * Create or update a tag in GitLab using the API
 *
 * This function needs to be async because it uses fetch which returns promises
 * and we need to handle request/response flows that can't be made synchronous.
 */
async function createOrUpdateTag(apiUrl, projectPath, tagName, message, ref, token, logger) {
  const headers = {
    'Content-Type': 'application/json',
    'PRIVATE-TOKEN': token
  };

  // Encode the project path and tag name for URL safety
  const encodedProjectPath = encodeURIComponent(projectPath);
  const encodedTagName = encodeURIComponent(tagName);

  let action = "create";

  try {
    // First check if the tag exists
    const checkUrl = `${apiUrl}/projects/${encodedProjectPath}/repository/tags/${encodedTagName}`;
    const checkResponse = await fetch(checkUrl, { headers });

    if (checkResponse.status >= 400 && checkResponse.status !== 404) {
      // Any error other than 404 (tag not found) is a real error
      let errorData = {};
      try {
        errorData = await checkResponse.json();
      } catch (e) {
        // If we can't parse the response as JSON, use the status text
        errorData = { message: checkResponse.statusText };
      }
      throw new SemanticReleaseError(`Failed to read tag ${tagName}: ${JSON.stringify(errorData)}`);
    }

    if (checkResponse.status === 200) {
      // Tag exists, update it
      logger.log(`Truncated tag ${tagName} already exists, updating it...`);
      action = "update";

      // In GitLab, to update a tag, we need to delete it and create again
      const deleteUrl = `${apiUrl}/projects/${encodedProjectPath}/repository/tags/${encodedTagName}`;
      const deleteResponse = await fetch(deleteUrl, {
        method: 'DELETE',
        headers
      });

      if (!deleteResponse.ok) {
        const deleteData = await deleteResponse.json();
        throw new SemanticReleaseError(`Failed to delete tag ${tagName}: ${JSON.stringify(deleteData)}`);
      }

      // Brief delay to ensure GitLab processes the deletion
      await new Promise(resolve => setTimeout(resolve, 1000));
    }

    // Create the tag
    const createUrl = `${apiUrl}/projects/${encodedProjectPath}/repository/tags`;
    const createResponse = await fetch(createUrl, {
      method: 'POST',
      headers,
      body: JSON.stringify({
        tag_name: tagName,
        ref: ref,
        message: message
      })
    });

    if (!createResponse.ok) {
      const createData = await createResponse.json();
      throw new SemanticReleaseError(`Failed to create tag ${tagName}: ${JSON.stringify(createData)}`);
    }

    logger.log(`Successfully ${action}d tag ${tagName}`);
  } catch (error) {
    logger.error(`Error creating or updating tag ${tagName}: ${error.message}`);
    throw error;
  }
}
